"use strict";
const tslib_1 = require("tslib");
const array_1 = require("@tsfun/array");
const fs_extra_1 = require("fs-extra");
const list_all_targets_1 = tslib_1.__importDefault(require("../list-all-targets"));
/**
 * Delete all compiled products corresponding to source files inside `root`
 * @param root Directory that contains all source files
 * @param options Options
 * @param options.deep A function that tells when to go deeper
 * @param options.isSource A function that tells which file is a source file
 * @param options.listTargets A function that lists compiled products corresponding to a source file
 * @returns An object
 *   * Property `targets`: An aggregation of `options.listTargets`'s return values
 *   * Property `reports`: An array of reports which tell whether or not deletion succeed
 *   * Property `success`: An array of paths to files which are successfully deleted
 *   * Property `failure`: An array of paths to files which are failed to be deleted
 */
async function clean(root, options) {
    const del = (file) => fs_extra_1.unlink(file).then(onUnlinkSuccess, onUnlinkFailure);
    const r2f = (reports) => reports.map(x => x.file);
    const onUnlinkSuccess = () => ({ success: true });
    const onUnlinkFailure = (error) => ({ success: false, error });
    const targets = await list_all_targets_1.default(root, options);
    const reports = await Promise.all(targets.map(async (file) => ({ file, deletion: await del(file) })));
    const [success, failure] = array_1.partition(reports, x => x.deletion.success);
    return { targets, reports, success: r2f(success), failure: r2f(failure) };
}
module.exports = clean;
//# sourceMappingURL=index.js.map